@@ -2,7 +2,6 @@
 require './test/test_helper'
 
 describe "Cron Job" do
-
   before do
     #clear all previous saved data from redis
     Sidekiq.redis do |conn|
@@ -257,26 +256,39 @@ describe "Cron Job" do
     end
   end
 
+  describe '#sidekiq_worker_message settings overwrite queue name' do
+    before do
+      @args = {
+        name:  'Test',
+        cron:  '* * * * *',
+        queue: 'super_queue',
+        klass: 'CronTestClassWithQueue',
+        args:  { foo: 'bar' }
+      }
+      @job = Sidekiq::Cron::Job.new(@args)
+    end
+
+    it 'should return valid payload for Sidekiq::Client with overwrite queue name' do
+      payload = {
+        "retry" => false,
+        "backtrace"=>true,
+        "queue" => "super_queue",
+        "class" => "CronTestClassWithQueue",
+        "args"  => [{:foo=>"bar"}]
+      }
+      assert_equal @job.sidekiq_worker_message, payload
+    end
+  end
+
   describe '#active_job_message' do
     before do
       SecureRandom.stubs(:uuid).returns('XYZ')
-      module ActiveJob
-        class Base
-          def self.queue_name_prefix
-            @@queue_name_prefix
-          end
-          def self.queue_name_prefix=(queue_name_prefix)
-            @@queue_name_prefix = queue_name_prefix
-          end
-        end
-      end
-      class ActiveJobTest < ActiveJob::Base; end
       ActiveJob::Base.queue_name_prefix = ''
 
       @args = {
         name:  'Test',
         cron:  '* * * * *',
-        klass: 'ActiveJobTest',
+        klass: 'ActiveJobCronTestClass',
         queue: 'super_queue',
         description: nil,
         args:  { foo: 'bar' }
@@ -290,7 +302,7 @@ describe "Cron Job" do
         'queue'       => 'super_queue',
         'description' => nil,
         'args'        => [{
-          'job_class'  => 'ActiveJobTest',
+          'job_class'  => 'ActiveJobCronTestClass',
           'job_id'     => 'XYZ',
           'queue_name' => 'super_queue',
           'arguments'  => [{foo: 'bar'}]
@@ -303,23 +315,12 @@ describe "Cron Job" do
   describe '#active_job_message with queue_name_prefix' do
     before do
       SecureRandom.stubs(:uuid).returns('XYZ')
-      module ActiveJob
-        class Base
-          def self.queue_name_prefix
-            @@queue_name_prefix
-          end
-          def self.queue_name_prefix=(queue_name_prefix)
-            @@queue_name_prefix = queue_name_prefix
-          end
-        end
-      end
-      class ActiveJobTest < ActiveJob::Base; end
       ActiveJob::Base.queue_name_prefix = "prefix"
 
       @args = {
         name:  'Test',
         cron:  '* * * * *',
-        klass: 'ActiveJobTest',
+        klass: 'ActiveJobCronTestClass',
         queue: 'super_queue',
         queue_name_prefix: 'prefix',
         args:  { foo: 'bar' }
@@ -333,7 +334,7 @@ describe "Cron Job" do
         'queue' => 'prefix_super_queue',
         'description' => nil,
         'args'  =>[{
-          'job_class'  => 'ActiveJobTest',
+          'job_class'  => 'ActiveJobCronTestClass',
           'job_id'     => 'XYZ',
           'queue_name' => 'prefix_super_queue',
           'arguments'  => [{foo: 'bar'}]
@@ -346,71 +347,45 @@ describe "Cron Job" do
   describe '#enque!' do
     describe 'active job' do
       before do
-        module ActiveJob
-          class Base
-            def self.queue_name_prefix
-              @@queue_name_prefix
-            end
-            def self.queue_name_prefix=(queue_name_prefix)
-              @@queue_name_prefix = queue_name_prefix
-            end
-          end
-        end
-        class ActiveJobTest < ActiveJob::Base; end
-        ActiveJob::Base.queue_name_prefix = ""
-
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest'
+          klass: 'ActiveJobCronTestClass'
         }
         @job = Sidekiq::Cron::Job.new(@args)
       end
 
       it 'pushes to queue active jobs message' do
-        @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [])
+        @job.expects(:enqueue_active_job)
+            .returns(true)
         @job.enque!
       end
     end
 
     describe 'active job with queue_name_prefix' do
       before do
-        module ActiveJob
-          class Base
-            def self.queue_name_prefix
-              @@queue_name_prefix
-            end
-            def self.queue_name_prefix=(queue_name_prefix)
-              @@queue_name_prefix = queue_name_prefix
-            end
-          end
-        end
-        class ActiveJobTest < ActiveJob::Base; end
-        ActiveJob::Base.queue_name_prefix = "prefix"
-
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest',
+          klass: 'ActiveJobCronTestClass',
           queue: 'cron'
         }
         @job = Sidekiq::Cron::Job.new(@args)
       end
 
       it 'pushes to queue active jobs message with queue_name_prefix' do
-        @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [], 'queue' => 'prefix_cron')
+        @job.expects(:enqueue_active_job)
+            .returns(true)
         @job.enque!
       end
     end
 
-    describe 'active job via configuration (bool: true)' do
+    describe 'active job via configuration (bool: true) [unknown class]' do
       before do
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest',
+          klass: 'UnknownClass',
           active_job: true
         }
         @job = Sidekiq::Cron::Job.new(@args)
@@ -418,17 +393,17 @@ describe "Cron Job" do
 
       it 'pushes to queue active jobs message' do
         @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [])
+          .returns('class' => 'UnknownClass', 'args' => [])
         @job.enque!
       end
     end
 
-    describe 'active job via configuration (string: true)' do
+    describe 'active job via configuration (string: true) [unknown class]' do
       before do
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest',
+          klass: 'UnknownClass',
           active_job: 'true'
         }
         @job = Sidekiq::Cron::Job.new(@args)
@@ -436,17 +411,17 @@ describe "Cron Job" do
 
       it 'pushes to queue active jobs message' do
         @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [])
+          .returns('class' => 'UnknownClass', 'args' => [])
         @job.enque!
       end
     end
 
-    describe 'active job via configuration (string: yes)' do
+    describe 'active job via configuration (string: yes) [unknown class]' do
       before do
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest',
+          klass: 'UnknownClass',
           active_job: 'yes'
         }
         @job = Sidekiq::Cron::Job.new(@args)
@@ -454,17 +429,17 @@ describe "Cron Job" do
 
       it 'pushes to queue active jobs message' do
         @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [])
+          .returns('class' => 'UnknownClass', 'args' => [])
         @job.enque!
       end
     end
 
-    describe 'active job via configuration (number: 1)' do
+    describe 'active job via configuration (number: 1) [unknown class]' do
       before do
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest',
+          klass: 'UnknownClass',
           active_job: 1
         }
         @job = Sidekiq::Cron::Job.new(@args)
@@ -472,17 +447,17 @@ describe "Cron Job" do
 
       it 'pushes to queue active jobs message' do
         @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [])
+          .returns('class' => 'UnknownClass', 'args' => [])
         @job.enque!
       end
     end
 
-    describe 'active job via configuration with queue_name_prefix option' do
+    describe 'active job via configuration with queue_name_prefix option [unknown class]' do
       before do
         @args = {
           name:  'Test',
           cron:  '* * * * *',
-          klass: 'ActiveJobTest',
+          klass: 'UnknownClass',
           queue: 'cron',
           active_job: true,
           queue_name_prefix: 'prefix'
@@ -492,7 +467,7 @@ describe "Cron Job" do
 
       it 'pushes to queue active jobs message with queue_name_prefix' do
         @job.expects(:active_job_message)
-          .returns('class' => 'Test', 'args' => [], 'queue' => 'prefix_cron')
+          .returns('class' => 'UnknownClass', 'args' => [], 'queue' => 'prefix_cron')
         @job.enque!
       end
     end
@@ -508,8 +483,8 @@ describe "Cron Job" do
       end
 
       it 'pushes to queue active jobs message' do
-        @job.expects(:sidekiq_worker_message)
-          .returns('class' => 'Test', 'args' => [])
+        @job.expects(:enqueue_sidekiq_worker)
+            .returns(true)
         @job.enque!
       end
     end
@@ -526,7 +501,8 @@ describe "Cron Job" do
       end
 
       it 'pushes to queue sidekiq worker message' do
-        assert_equal @job.sidekiq_worker_message, { 'class' => 'UnknownClass', 'args' => [], 'queue' => 'another' }
+        @job.expects(:sidekiq_worker_message)
+            .returns('class' => 'UnknownClass', 'args' => [], 'queue' => 'another')
         @job.enque!
       end
     end
@@ -835,6 +811,16 @@ describe "Cron Job" do
       refute Sidekiq::Cron::Job.new(@args.merge(cron: "0 1,13 * * *")).should_enque? @time
     end
 
+    it 'doesnt skip enqueuing if job is resaved near next enqueue time' do
+      job = Sidekiq::Cron::Job.new(@args)
+      assert job.test_and_enque_for_time!(@time), "should enqueue"
+
+      future_now = @time + 1 * 60 * 60
+      Time.stubs(:now).returns(future_now) # save uses Time.now
+      job.save
+      assert Sidekiq::Cron::Job.new(@args).test_and_enque_for_time!(future_now + 30), "should enqueue"
+    end
+
     it "remove old enque times + should be enqeued" do
       job = Sidekiq::Cron::Job.new(@args)
       assert_nil job.last_enqueue_time
@@ -843,7 +829,7 @@ describe "Cron Job" do
 
       refute Sidekiq::Cron::Job.new(@args).test_and_enque_for_time!(@time), "should not enqueue"
       Sidekiq.redis do |conn|
-        assert_equal conn.zcard(Sidekiq::Cron::Job.new(@args).send(:job_enqueued_key)), 2, "Should have two enqueued job (first was in save, second in enque)"
+        assert_equal conn.zcard(Sidekiq::Cron::Job.new(@args).send(:job_enqueued_key)), 1, "Should have one enqueued job"
       end
       assert_equal Sidekiq::Queue.all.first.size, 1, "Sidekiq queue 1 job in queue"
 
@@ -852,7 +838,7 @@ describe "Cron Job" do
       refute Sidekiq::Cron::Job.new(@args).test_and_enque_for_time! @time + 1 * 60 * 60
 
       Sidekiq.redis do |conn|
-        assert_equal conn.zcard(Sidekiq::Cron::Job.new(@args).send(:job_enqueued_key)), 3, "Should have two enqueued job + one from start"
+        assert_equal conn.zcard(Sidekiq::Cron::Job.new(@args).send(:job_enqueued_key)), 2, "Should have two enqueued job"
       end
       assert_equal Sidekiq::Queue.all.first.size, 2, "Sidekiq queue 2 jobs in queue"
 
@@ -949,5 +935,37 @@ describe "Cron Job" do
         assert_equal Sidekiq::Cron::Job.all.size, 2, "Should have 2 jobs after load"
       end
     end
+
+    describe "from array with queue_name" do
+      before do
+        @jobs_array = [
+          {
+            'name'  => 'name_of_job',
+            'class' => 'CronTestClassWithQueue',
+            'cron'  => '1 * * * *',
+            'args'  => '(OPTIONAL) [Array or Hash]',
+            'queue' => 'from_array'
+          }
+        ]
+      end
+
+      it "create new jobs and update old one with same settings" do
+        assert_equal Sidekiq::Cron::Job.all.size, 0, "Should have 0 jobs before load"
+        out = Sidekiq::Cron::Job.load_from_array @jobs_array
+        assert_equal out.size, 0, "should have 0 error"
+        assert_equal Sidekiq::Cron::Job.all.size, 1, "Should have 2 jobs after load"
+
+        payload = {
+          "retry" => false,
+          "backtrace"=>true,
+          "queue" => "from_array",
+          "class" => "CronTestClassWithQueue",
+          "args"  => ['(OPTIONAL) [Array or Hash]']
+        }
+
+        assert_equal Sidekiq::Cron::Job.all.first.sidekiq_worker_message, payload
+      end
+
+    end
   end
 end
